----------------------------------------------------------------------------------
-- Company:     Cesys GmbH
-- Engineer:    Vinay Visweswara (vinay.visweswara@cesys.com)
-- 
-- Create Date: 25th November, 2016 11:10:14 AM
-- Design Name: AXI4-USB3.0 Interface
-- Module Name: axi_fx3_if - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity axi_fx3_if is
Port(
      sys_clk               : in STD_LOGIC;
      i_reset               : in STD_LOGIC;
      -- FX3 interface
      i_fx3_pclk            : in STD_LOGIC;
      o_fx3_pclk            : out STD_LOGIC;
      i_fx3_flag            : in STD_LOGIC_VECTOR (3 downto 0);
      o_fx3_slcs_n          : out STD_LOGIC;
      o_fx3_slrd_n          : out STD_LOGIC;
      o_fx3_slwr_n          : out STD_LOGIC;
      o_fx3_sloe_n          : out STD_LOGIC;
      o_fx3_pktend_n        : out STD_LOGIC;
      o_fx3_faddr           : out STD_LOGIC_VECTOR (1 downto 0);
      io_fx3_fdata          : inout STD_LOGIC_VECTOR (31 downto 0);
      
      -- Read Channel data interface      : AXI4-Full (Read Only)
      M_AXI_araddr   : out STD_LOGIC_VECTOR ( 31 downto 0 );
      M_AXI_arburst  : out STD_LOGIC_VECTOR ( 1 downto 0 );
      M_AXI_arcache  : out STD_LOGIC_VECTOR ( 3 downto 0 );
      M_AXI_arlen    : out STD_LOGIC_VECTOR ( 7 downto 0 );
      M_AXI_arprot   : out STD_LOGIC_VECTOR ( 2 downto 0 );
      M_AXI_arready  : in STD_LOGIC;
      M_AXI_arsize   : out STD_LOGIC_VECTOR ( 2 downto 0 );
      M_AXI_aruser   : out STD_LOGIC_VECTOR ( 3 downto 0 );
      M_AXI_arvalid  : out STD_LOGIC;
      M_AXI_rdata    : in STD_LOGIC_VECTOR ( 31 downto 0 );
      M_AXI_rlast    : in STD_LOGIC;
      M_AXI_rready   : out STD_LOGIC;
      M_AXI_rresp    : in STD_LOGIC_VECTOR ( 1 downto 0 );
      M_AXI_rvalid   : in STD_LOGIC;

      -- Write Channel data interface     : AXI4-Full (Read Only)
      M_AXI_awaddr   : out STD_LOGIC_VECTOR ( 31 downto 0 );
      M_AXI_awburst  : out STD_LOGIC_VECTOR ( 1 downto 0 );
      M_AXI_awcache  : out STD_LOGIC_VECTOR ( 3 downto 0 );
      M_AXI_awlen    : out STD_LOGIC_VECTOR ( 7 downto 0 );
      M_AXI_awprot   : out STD_LOGIC_VECTOR ( 2 downto 0 );
      M_AXI_awready  : in STD_LOGIC;
      M_AXI_awsize   : out STD_LOGIC_VECTOR ( 2 downto 0 );
      M_AXI_awuser   : out STD_LOGIC_VECTOR ( 3 downto 0 );
      M_AXI_awvalid  : out STD_LOGIC;
      M_AXI_bready   : out STD_LOGIC;
      M_AXI_bresp    : in STD_LOGIC_VECTOR ( 1 downto 0 );
      M_AXI_bvalid   : in STD_LOGIC;
      M_AXI_wdata    : out STD_LOGIC_VECTOR ( 31 downto 0 );
      M_AXI_wlast    : out STD_LOGIC;
      M_AXI_wready   : in STD_LOGIC;
      M_AXI_wstrb    : out STD_LOGIC_VECTOR ( 3 downto 0 );
      M_AXI_wvalid   : out STD_LOGIC;

      o_RD_ERR              : out STD_LOGIC;
      o_WR_ERR              : out STD_LOGIC;
      
      S_S2MM_STATUS         : out STD_LOGIC_VECTOR ( 7 downto 0 );
      S_MM2S_STATUS         : out STD_LOGIC_VECTOR ( 7 downto 0 );

      -- GTP Ports  : AXI Stream
      S_gt0_master0_tdata   : out STD_LOGIC_VECTOR (31 downto 0);
      S_gt0_master0_tkeep   : out STD_LOGIC_VECTOR (3 downto 0);
      S_gt0_master0_tvalid  : out STD_LOGIC;
      S_gt0_master0_tlast   : out STD_LOGIC;
      S_gt0_master0_tready  : in  STD_LOGIC;

      S_gt0_slave0_tdata    : in  STD_LOGIC_VECTOR (31 downto 0);
      S_gt0_slave0_tkeep    : in  STD_LOGIC_VECTOR (3 downto 0);
      S_gt0_slave0_tvalid   : in  STD_LOGIC;
      S_gt0_slave0_tlast    : in  STD_LOGIC;
      S_gt0_slave0_tready   : out STD_LOGIC
      
      -- o_debug_port   : out STD_LOGIC_VECTOR( 11 downto 0)
);
end axi_fx3_if;

architecture Behavioral of axi_fx3_if is

component fx3_axis_dma_interface is
Port( 
      sys_clk               : in STD_LOGIC;
      o_fx3_pclk            : out STD_LOGIC;
      i_reset               : in STD_LOGIC;

      -- FX3 interface
      i_fx3_pclk            : in STD_LOGIC;
      i_fx3_flag            : in STD_LOGIC_VECTOR (3 downto 0);
      o_fx3_slcs_n          : out STD_LOGIC;
      o_fx3_slrd_n          : out STD_LOGIC;
      o_fx3_slwr_n          : out STD_LOGIC;
      o_fx3_sloe_n          : out STD_LOGIC;
      o_fx3_pktend_n        : out STD_LOGIC;
      o_fx3_faddr           : out STD_LOGIC_VECTOR (1 downto 0);
      io_fx3_fdata          : inout STD_LOGIC_VECTOR (31 downto 0);

      -- AXI4-Stream MM2S Command interface
      mm2s_cmd_tdata        : out STD_LOGIC_VECTOR(71 downto 0);
      mm2s_cmd_tvalid       : out STD_LOGIC;
      mm2s_cmd_tready       : in STD_LOGIC;

      -- AXI4-Stream Read-Data-from-Peripheral FIFO Interface
      M_AXIS_MM2S_tdata     : in  STD_LOGIC_VECTOR ( 31 downto 0 );
      M_AXIS_MM2S_tkeep     : in  STD_LOGIC_VECTOR ( 3 downto 0 );
      M_AXIS_MM2S_tlast     : in  STD_LOGIC;
      M_AXIS_MM2S_tready    : out STD_LOGIC;
      M_AXIS_MM2S_tvalid    : in  STD_LOGIC;

      -- AXI4-Stream S2MM Command interface
      s2mm_cmd_tdata        : out STD_LOGIC_VECTOR(71 downto 0);
      s2mm_cmd_tvalid       : out STD_LOGIC;
      s2mm_cmd_tready       : in STD_LOGIC;
      
      s2mm_addr_req_posted    : in  STD_LOGIC;
      s2mm_allow_addr_req     : out STD_LOGIC;
      s2mm_wr_xfer_cmplt      : in  STD_LOGIC;

      -- AXI4-Stream Write-Data-to-Peripheral FIFO Interface
      S_AXIS_S2MM_tdata     : out STD_LOGIC_VECTOR ( 31 downto 0 );
      S_AXIS_S2MM_tkeep     : out STD_LOGIC_VECTOR ( 3 downto 0 );
      S_AXIS_S2MM_tlast     : out STD_LOGIC;
      S_AXIS_S2MM_tready    : in  STD_LOGIC;
      S_AXIS_S2MM_tvalid    : out STD_LOGIC;
      
      M_AXIS_GTP_tdata      : out STD_LOGIC_VECTOR (31 downto 0);
      M_AXIS_GTP_tkeep      : out STD_LOGIC_VECTOR (3 downto 0);
      M_AXIS_GTP_tvalid     : out STD_LOGIC;
      M_AXIS_GTP_tlast      : out STD_LOGIC;
      M_AXIS_GTP_tready     : in  STD_LOGIC;
      -- AXIS_GTP_select       : out STD_LOGIC_VECTOR(1 downto 0);

      S_AXIS_GTP_tdata      : in  STD_LOGIC_VECTOR (31 downto 0);
      S_AXIS_GTP_tkeep      : in  STD_LOGIC_VECTOR (3 downto 0);
      S_AXIS_GTP_tvalid     : in  STD_LOGIC;
      S_AXIS_GTP_tlast      : in  STD_LOGIC;
      S_AXIS_GTP_tready     : out STD_LOGIC

      -- o_debug_port          : out STD_LOGIC_VECTOR ( 11 downto 0 )
    );
end component;

signal s_clk : STD_LOGIC;
-- signals connecting "fx3_axis_dma_interface" and "data_mover_wrapper" --
------------------------------------------------------------------------------
-- AXI4-Stream MM2S Command interface
signal mm2s_cmd_tdata      : STD_LOGIC_VECTOR(71 downto 0);
signal mm2s_cmd_tvalid     : STD_LOGIC;
signal mm2s_cmd_tready     : STD_LOGIC;

-- AXI4-Stream Read-Data-from-Peripheral FIFO Interface
signal axis_mm2s_tdata     : STD_LOGIC_VECTOR ( 31 downto 0 );
signal axis_mm2s_tkeep     : STD_LOGIC_VECTOR ( 3 downto 0 );
signal axis_mm2s_tlast     : STD_LOGIC;
signal axis_mm2s_tready    : STD_LOGIC;
signal axis_mm2s_tvalid    : STD_LOGIC;

-- AXI4-Stream S2MM Command interface
signal s2mm_cmd_tdata      : STD_LOGIC_VECTOR(71 downto 0);
signal s2mm_cmd_tvalid     : STD_LOGIC;
signal s2mm_cmd_tready     : STD_LOGIC;

signal s_s2mm_addr_req_posted    : STD_LOGIC;
signal s_s2mm_allow_addr_req     : STD_LOGIC;
signal s_s2mm_wr_xfer_cmplt      : STD_LOGIC;

-- AXI4-Stream Write-Data-to-Peripheral FIFO Interface
signal axis_s2mm_tdata     : STD_LOGIC_VECTOR ( 31 downto 0 );
signal axis_s2mm_tkeep     : STD_LOGIC_VECTOR ( 3 downto 0 );
signal axis_s2mm_tlast     : STD_LOGIC;
signal axis_s2mm_tready    : STD_LOGIC;
signal axis_s2mm_tvalid    : STD_LOGIC;

signal s_S2MM_STS_tvalid  : STD_LOGIC := '0';
signal s_MM2S_STS_tvalid  : STD_LOGIC := '0';
signal s_STS_tready       : STD_LOGIC := '1';

------------------------------------
component data_mover_wrapper is
port
  (
    -- Global
    clk_in : in STD_LOGIC;
    reset : in STD_LOGIC;

    -- Read Channel Command interface   : AXI4-Stream
    S_AXIS_MM2S_CMD_tdata   : in STD_LOGIC_VECTOR ( 71 downto 0 );
    S_AXIS_MM2S_CMD_tready  : out STD_LOGIC;
    S_AXIS_MM2S_CMD_tvalid  : in STD_LOGIC;
    mm2s_err : out STD_LOGIC;

    -- Read Data interface             : AXI4-Stream
    M_AXIS_MM2S_tdata   : out STD_LOGIC_VECTOR ( 31 downto 0 );
    M_AXIS_MM2S_tkeep   : out STD_LOGIC_VECTOR ( 3 downto 0 );
    M_AXIS_MM2S_tlast   : out STD_LOGIC;
    M_AXIS_MM2S_tready  : in STD_LOGIC;
    M_AXIS_MM2S_tvalid  : out STD_LOGIC;

    -- Read Channel data interface      : AXI4-Full (Read Only) to AXI Interconnect
    M_AXI_MM2S_araddr   : out STD_LOGIC_VECTOR ( 31 downto 0 );
    M_AXI_MM2S_arburst  : out STD_LOGIC_VECTOR ( 1 downto 0 );
    M_AXI_MM2S_arcache  : out STD_LOGIC_VECTOR ( 3 downto 0 );
    M_AXI_MM2S_arlen    : out STD_LOGIC_VECTOR ( 7 downto 0 );
    M_AXI_MM2S_arprot   : out STD_LOGIC_VECTOR ( 2 downto 0 );
    M_AXI_MM2S_arready  : in STD_LOGIC;
    M_AXI_MM2S_arsize   : out STD_LOGIC_VECTOR ( 2 downto 0 );
    M_AXI_MM2S_aruser   : out STD_LOGIC_VECTOR ( 3 downto 0 );
    M_AXI_MM2S_arvalid  : out STD_LOGIC;
    M_AXI_MM2S_rdata    : in STD_LOGIC_VECTOR ( 31 downto 0 );
    M_AXI_MM2S_rlast    : in STD_LOGIC;
    M_AXI_MM2S_rready   : out STD_LOGIC;
    M_AXI_MM2S_rresp    : in STD_LOGIC_VECTOR ( 1 downto 0 );
    M_AXI_MM2S_rvalid   : in STD_LOGIC;
    
    M_AXIS_MM2S_STS_tdata : out STD_LOGIC_VECTOR ( 7 downto 0 );
    M_AXIS_MM2S_STS_tkeep : out STD_LOGIC_VECTOR ( 0 to 0 );
    M_AXIS_MM2S_STS_tlast : out STD_LOGIC;
    M_AXIS_MM2S_STS_tready : in STD_LOGIC;
    M_AXIS_MM2S_STS_tvalid : out STD_LOGIC;

    -- Write Channel Command interface   : AXI4-Stream
    S_AXIS_S2MM_CMD_tdata   : in STD_LOGIC_VECTOR ( 71 downto 0 );
    S_AXIS_S2MM_CMD_tready  : out STD_LOGIC;
    S_AXIS_S2MM_CMD_tvalid  : in STD_LOGIC;
    s2mm_err : out STD_LOGIC;
    s2mm_addr_req_posted    : out STD_LOGIC;
    s2mm_allow_addr_req     : in STD_LOGIC;
    s2mm_wr_xfer_cmplt      : out STD_LOGIC;

    -- Write Channel data interface     : AXI4-Full (Read Only) to AXI Interconnect
    M_AXI_S2MM_awaddr   : out STD_LOGIC_VECTOR ( 31 downto 0 );
    M_AXI_S2MM_awburst  : out STD_LOGIC_VECTOR ( 1 downto 0 );
    M_AXI_S2MM_awcache  : out STD_LOGIC_VECTOR ( 3 downto 0 );
    M_AXI_S2MM_awlen    : out STD_LOGIC_VECTOR ( 7 downto 0 );
    M_AXI_S2MM_awprot   : out STD_LOGIC_VECTOR ( 2 downto 0 );
    M_AXI_S2MM_awready  : in STD_LOGIC;
    M_AXI_S2MM_awsize   : out STD_LOGIC_VECTOR ( 2 downto 0 );
    M_AXI_S2MM_awuser   : out STD_LOGIC_VECTOR ( 3 downto 0 );
    M_AXI_S2MM_awvalid  : out STD_LOGIC;
    M_AXI_S2MM_bready   : out STD_LOGIC;
    M_AXI_S2MM_bresp    : in STD_LOGIC_VECTOR ( 1 downto 0 );
    M_AXI_S2MM_bvalid   : in STD_LOGIC;
    M_AXI_S2MM_wdata    : out STD_LOGIC_VECTOR ( 31 downto 0 );
    M_AXI_S2MM_wlast    : out STD_LOGIC;
    M_AXI_S2MM_wready   : in STD_LOGIC;
    M_AXI_S2MM_wstrb    : out STD_LOGIC_VECTOR ( 3 downto 0 );
    M_AXI_S2MM_wvalid   : out STD_LOGIC;
    
    -- Write Data interface             : AXI4-Stream
    S_AXIS_S2MM_tdata   : in STD_LOGIC_VECTOR ( 31 downto 0 );
    S_AXIS_S2MM_tkeep   : in STD_LOGIC_VECTOR ( 3 downto 0 );
    S_AXIS_S2MM_tlast   : in STD_LOGIC;
    S_AXIS_S2MM_tready  : out STD_LOGIC;
    S_AXIS_S2MM_tvalid  : in STD_LOGIC;
    
    M_AXIS_S2MM_STS_tdata : out STD_LOGIC_VECTOR ( 7 downto 0 );
    M_AXIS_S2MM_STS_tkeep : out STD_LOGIC_VECTOR ( 0 to 0 );
    M_AXIS_S2MM_STS_tlast : out STD_LOGIC;
    M_AXIS_S2MM_STS_tready : in STD_LOGIC;
    M_AXIS_S2MM_STS_tvalid : out STD_LOGIC
  );
end component;

-- type arr_std32 is array(natural range <>) of std_logic_vector(31 downto 0);
-- type arr_std4  is array(natural range <>) of std_logic_vector(3 downto 0);

-- signal s_gtp_master_tdata_arr  : arr_std32 (3 downto 0);   -- 4 std_logic_vector(31 downto 0)s
-- signal s_gtp_master_tkeep_arr  : arr_std4  (3 downto 0);   -- 4 std_logic_vector(3 downto 0)s
-- signal s_gtp_master_tlast_arr  : std_logic_vector(3 downto 0);
-- signal s_gtp_master_tready_arr : std_logic_vector(3 downto 0);
-- signal s_gtp_master_tvalid_arr : std_logic_vector(3 downto 0);

-- signal s_gtp_slave_tdata_arr  : arr_std32 (3 downto 0);   -- 4 std_logic_vector(31 downto 0)s
-- signal s_gtp_slave_tkeep_arr  : arr_std4  (3 downto 0);   -- 4 std_logic_vector(3 downto 0)s
-- signal s_gtp_slave_tlast_arr  : std_logic_vector(3 downto 0);
-- signal s_gtp_slave_tready_arr : std_logic_vector(3 downto 0);
-- signal s_gtp_slave_tvalid_arr : std_logic_vector(3 downto 0);

-- signal s_AXIS_GTP_select  : STD_LOGIC_VECTOR ( 1 downto 0 );

signal s_GTP_Master_tdata   : STD_LOGIC_VECTOR ( 31 downto 0 );
signal s_GTP_Master_tkeep   : STD_LOGIC_VECTOR ( 3 downto 0 );
signal s_GTP_Master_tlast   : STD_LOGIC;
signal s_GTP_Master_tready  : STD_LOGIC;
signal s_GTP_Master_tvalid  : STD_LOGIC;

signal s_GTP_Slave_tdata  : STD_LOGIC_VECTOR ( 31 downto 0 );
signal s_GTP_Slave_tkeep  : STD_LOGIC_VECTOR ( 3 downto 0 );
signal s_GTP_Slave_tlast  : STD_LOGIC;
signal s_GTP_Slave_tready : STD_LOGIC;
signal s_GTP_Slave_tvalid : STD_LOGIC;

begin

-- top port
-- FX3 Slave FIFO Interface timing is very critical..!!

-- The "o_fx3_pclk" is the one that drives the FX3 slave FIFOs.
-- In EFM03-Hardware, this clock is looped back to FPGA as an
-- input clock "i_fx3_pclk". This is done so that we can control
-- the input clock by adding appropriate delays to inputs in this
-- clock domain. It is not possible to control an output clock.
-----------------------------------------------------------------------
o_fx3_pclk <= s_clk;
-----------------------------------------------------------------------

-- Instantiations --
-----------------------------------------------------------------------
inst_data_mover:
component data_mover_wrapper port map
(
    -- Global
    clk_in  => sys_clk,
    reset   => i_reset,

    -- Read Channel Command interface : AXI4-Stream
    S_AXIS_MM2S_CMD_tdata   => mm2s_cmd_tdata ,
    S_AXIS_MM2S_CMD_tvalid  => mm2s_cmd_tvalid,
    S_AXIS_MM2S_CMD_tready  => mm2s_cmd_tready,
    mm2s_err => o_RD_ERR,
    -- Read Data interface : AXI4-Stream
    M_AXIS_MM2S_tdata   => axis_mm2s_tdata ,
    M_AXIS_MM2S_tkeep   => axis_mm2s_tkeep ,
    M_AXIS_MM2S_tlast   => axis_mm2s_tlast ,
    M_AXIS_MM2S_tready  => axis_mm2s_tready,
    M_AXIS_MM2S_tvalid  => axis_mm2s_tvalid,

    -- Read Channel data interface : AXI4-Full (Read Only) to AXI Interconnect
    M_AXI_MM2S_araddr   => M_AXI_araddr ,
    M_AXI_MM2S_arburst  => M_AXI_arburst,
    M_AXI_MM2S_arcache  => M_AXI_arcache,
    M_AXI_MM2S_arlen    => M_AXI_arlen  ,
    M_AXI_MM2S_arprot   => M_AXI_arprot ,
    M_AXI_MM2S_arready  => M_AXI_arready,
    M_AXI_MM2S_arsize   => M_AXI_arsize ,
    M_AXI_MM2S_aruser   => M_AXI_aruser ,
    M_AXI_MM2S_arvalid  => M_AXI_arvalid,
    M_AXI_MM2S_rdata    => M_AXI_rdata  ,
    M_AXI_MM2S_rlast    => M_AXI_rlast  ,
    M_AXI_MM2S_rready   => M_AXI_rready ,
    M_AXI_MM2S_rresp    => M_AXI_rresp  ,
    M_AXI_MM2S_rvalid   => M_AXI_rvalid ,

    -- Write Channel Command interface : AXI4-Stream
    S_AXIS_S2MM_CMD_tdata  => s2mm_cmd_tdata ,
    S_AXIS_S2MM_CMD_tvalid => s2mm_cmd_tvalid,
    S_AXIS_S2MM_CMD_tready => s2mm_cmd_tready,
    s2mm_err               => o_WR_ERR,
    s2mm_addr_req_posted   => s_s2mm_addr_req_posted,
    s2mm_allow_addr_req    => s_s2mm_allow_addr_req ,
    s2mm_wr_xfer_cmplt     => s_s2mm_wr_xfer_cmplt  ,
    -- Write Data interface : AXI4-Stream
    S_AXIS_S2MM_tdata   => axis_s2mm_tdata ,
    S_AXIS_S2MM_tkeep   => axis_s2mm_tkeep ,
    S_AXIS_S2MM_tlast   => axis_s2mm_tlast ,
    S_AXIS_S2MM_tready  => axis_s2mm_tready,
    S_AXIS_S2MM_tvalid  => axis_s2mm_tvalid,

    -- Write Channel data interface : AXI4-Full (Read Only) to AXI Interconnect
    M_AXI_S2MM_awaddr   => M_AXI_awaddr ,
    M_AXI_S2MM_awburst  => M_AXI_awburst,
    M_AXI_S2MM_awcache  => M_AXI_awcache,
    M_AXI_S2MM_awlen    => M_AXI_awlen  ,
    M_AXI_S2MM_awprot   => M_AXI_awprot ,
    M_AXI_S2MM_awready  => M_AXI_awready,
    M_AXI_S2MM_awsize   => M_AXI_awsize ,
    M_AXI_S2MM_awuser   => M_AXI_awuser ,
    M_AXI_S2MM_awvalid  => M_AXI_awvalid,
    M_AXI_S2MM_bready   => M_AXI_bready ,
    M_AXI_S2MM_bresp    => M_AXI_bresp  ,
    M_AXI_S2MM_bvalid   => M_AXI_bvalid ,
    M_AXI_S2MM_wdata    => M_AXI_wdata  ,
    M_AXI_S2MM_wlast    => M_AXI_wlast  ,
    M_AXI_S2MM_wready   => M_AXI_wready ,
    M_AXI_S2MM_wstrb    => M_AXI_wstrb  ,
    M_AXI_S2MM_wvalid   => M_AXI_wvalid ,

    -- Status signals
    M_AXIS_MM2S_STS_tdata  => S_MM2S_STATUS,
    M_AXIS_MM2S_STS_tkeep  => open,
    M_AXIS_MM2S_STS_tlast  => open,
    M_AXIS_MM2S_STS_tready => s_STS_tready,
    M_AXIS_MM2S_STS_tvalid => s_MM2S_STS_tvalid,

    M_AXIS_S2MM_STS_tdata   => S_S2MM_STATUS,
    M_AXIS_S2MM_STS_tkeep   => open,
    M_AXIS_S2MM_STS_tlast   => open,
    M_AXIS_S2MM_STS_tready  => s_STS_tready,
    M_AXIS_S2MM_STS_tvalid  => s_S2MM_STS_tvalid
);

inst_FX3_axis_dma:
component fx3_axis_dma_interface
port map
(
  sys_clk               => sys_clk,
  i_reset               => i_reset        ,
  -- FX3 interface      => 
  i_fx3_pclk            => i_fx3_pclk    ,
  o_fx3_pclk            => s_clk         ,
  i_fx3_flag            => i_fx3_flag    ,
  o_fx3_slcs_n          => o_fx3_slcs_n  ,
  o_fx3_slrd_n          => o_fx3_slrd_n  ,
  o_fx3_slwr_n          => o_fx3_slwr_n  ,
  o_fx3_sloe_n          => o_fx3_sloe_n  ,
  o_fx3_pktend_n        => o_fx3_pktend_n,
  o_fx3_faddr           => o_fx3_faddr   ,
  io_fx3_fdata          => io_fx3_fdata  ,

  mm2s_cmd_tdata        => mm2s_cmd_tdata  ,
  mm2s_cmd_tvalid       => mm2s_cmd_tvalid ,
  mm2s_cmd_tready       => mm2s_cmd_tready ,

  M_AXIS_MM2S_tdata     => axis_mm2s_tdata ,
  M_AXIS_MM2S_tkeep     => axis_mm2s_tkeep ,
  M_AXIS_MM2S_tlast     => axis_mm2s_tlast ,
  M_AXIS_MM2S_tready    => axis_mm2s_tready,
  M_AXIS_MM2S_tvalid    => axis_mm2s_tvalid,

  s2mm_cmd_tdata        => s2mm_cmd_tdata  ,
  s2mm_cmd_tvalid       => s2mm_cmd_tvalid ,
  s2mm_cmd_tready       => s2mm_cmd_tready ,
  
  s2mm_addr_req_posted  => s_s2mm_addr_req_posted,
  s2mm_allow_addr_req   => s_s2mm_allow_addr_req ,
  s2mm_wr_xfer_cmplt    => s_s2mm_wr_xfer_cmplt  ,

  S_AXIS_S2MM_tdata     => axis_s2mm_tdata ,
  S_AXIS_S2MM_tkeep     => axis_s2mm_tkeep ,
  S_AXIS_S2MM_tlast     => axis_s2mm_tlast ,
  S_AXIS_S2MM_tready    => axis_s2mm_tready,
  S_AXIS_S2MM_tvalid    => axis_s2mm_tvalid,

  M_AXIS_GTP_tdata      => s_gtp_Master_tdata ,
  M_AXIS_GTP_tkeep      => s_gtp_Master_tkeep ,
  M_AXIS_GTP_tlast      => s_gtp_Master_tlast ,
  M_AXIS_GTP_tvalid     => s_gtp_Master_tvalid,
  M_AXIS_GTP_tready     => s_gtp_Master_tready,

  -- AXIS_GTP_select       => s_AXIS_GTP_select,

  S_AXIS_GTP_tdata      => s_GTP_Slave_tdata ,
  S_AXIS_GTP_tkeep      => s_GTP_Slave_tkeep ,
  S_AXIS_GTP_tlast      => s_GTP_Slave_tlast ,
  S_AXIS_GTP_tvalid     => s_GTP_Slave_tvalid,
  S_AXIS_GTP_tready     => s_GTP_Slave_tready

  --o_debug_port          => o_debug_port
);
------------------------------------

-- process (s_clk) begin
  -- for i in 0 to 3 loop
    -- if i = to_integer(unsigned(s_AXIS_GTP_select)) then
      -- s_gtp_master_tdata_arr (i) <= s_gtp_Master_tdata;
      -- s_gtp_master_tkeep_arr (i) <= s_gtp_Master_tkeep;
      -- s_gtp_master_tlast_arr (i) <= s_gtp_Master_tlast;
      -- s_gtp_master_tvalid_arr(i) <= s_gtp_Master_tvalid;
      -- s_gtp_Master_tready <= s_gtp_master_tready_arr(i);
      
      -- s_GTP_Slave_tdata   <= s_gtp_slave_tdata_arr (i);
      -- s_GTP_Slave_tkeep   <= s_gtp_slave_tkeep_arr (i);
      -- s_GTP_Slave_tlast   <= s_gtp_slave_tlast_arr (i);
      -- s_GTP_Slave_tvalid  <= s_gtp_slave_tvalid_arr(i);
      -- s_gtp_slave_tready_arr(i)  <= s_GTP_Slave_tready;
    -- end if;
  -- end loop;
-- end process;


      S_gt0_master0_tdata   <= s_gtp_Master_tdata;
      S_gt0_master0_tkeep   <= s_gtp_Master_tkeep;
      S_gt0_master0_tlast   <= s_gtp_Master_tlast;
      S_gt0_master0_tvalid  <= s_gtp_Master_tvalid;
      s_gtp_Master_tready   <= S_gt0_master0_tready;
      
      s_GTP_Slave_tdata   <= S_gt0_slave0_tdata ;
      s_GTP_Slave_tkeep   <= S_gt0_slave0_tkeep ;
      s_GTP_Slave_tlast   <= S_gt0_slave0_tlast ;
      s_GTP_Slave_tvalid  <= S_gt0_slave0_tvalid;
      S_gt0_slave0_tready <= s_GTP_Slave_tready;

      -- S_gt0_master0_tdata         <= s_gtp_master_tdata_arr (0);
      -- S_gt0_master0_tkeep         <= s_gtp_master_tkeep_arr (0);
      -- S_gt0_master0_tlast         <= s_gtp_master_tlast_arr (0);
      -- S_gt0_master0_tvalid        <= s_gtp_master_tvalid_arr(0);
      -- s_gtp_master_tready_arr(0)  <= S_gt0_master0_tready;
      
      -- S_gt0_master1_tdata         <= s_gtp_master_tdata_arr (1);
      -- S_gt0_master1_tkeep         <= s_gtp_master_tkeep_arr (1);
      -- S_gt0_master1_tlast         <= s_gtp_master_tlast_arr (1);
      -- S_gt0_master1_tvalid        <= s_gtp_master_tvalid_arr(1);
      -- s_gtp_master_tready_arr(1)  <= S_gt0_master1_tready;
      
      -- S_gt0_master2_tdata         <= s_gtp_master_tdata_arr (2);
      -- S_gt0_master2_tkeep         <= s_gtp_master_tkeep_arr (2);
      -- S_gt0_master2_tlast         <= s_gtp_master_tlast_arr (2);
      -- S_gt0_master2_tvalid        <= s_gtp_master_tvalid_arr(2);
      -- s_gtp_master_tready_arr(2)  <= S_gt0_master2_tready;
      
      -- S_gt0_master3_tdata         <= s_gtp_master_tdata_arr (3);
      -- S_gt0_master3_tkeep         <= s_gtp_master_tkeep_arr (3);
      -- S_gt0_master3_tlast         <= s_gtp_master_tlast_arr (3);
      -- S_gt0_master3_tvalid        <= s_gtp_master_tvalid_arr(3);
      -- s_gtp_master_tready_arr(3)  <= S_gt0_master3_tready;
      
      -- s_gtp_slave_tdata_arr (0) <= S_gt0_slave0_tdata ;
      -- s_gtp_slave_tkeep_arr (0) <= S_gt0_slave0_tkeep ;
      -- s_gtp_slave_tlast_arr (0) <= S_gt0_slave0_tlast ;
      -- s_gtp_slave_tvalid_arr(0) <= S_gt0_slave0_tvalid;
      -- S_gt0_slave0_tready <= s_gtp_slave_tready_arr(0);
      
      -- s_gtp_slave_tdata_arr (1) <= S_gt0_slave1_tdata ;
      -- s_gtp_slave_tkeep_arr (1) <= S_gt0_slave1_tkeep ;
      -- s_gtp_slave_tlast_arr (1) <= S_gt0_slave1_tlast ;
      -- s_gtp_slave_tvalid_arr(1) <= S_gt0_slave1_tvalid;
      -- S_gt0_slave1_tready <= s_gtp_slave_tready_arr(1);

      -- s_gtp_slave_tdata_arr (2) <= S_gt0_slave2_tdata ;
      -- s_gtp_slave_tkeep_arr (2) <= S_gt0_slave2_tkeep ;
      -- s_gtp_slave_tlast_arr (2) <= S_gt0_slave2_tlast ;
      -- s_gtp_slave_tvalid_arr(2) <= S_gt0_slave2_tvalid;
      -- S_gt0_slave2_tready <= s_gtp_slave_tready_arr(2);

      -- s_gtp_slave_tdata_arr (3) <= S_gt0_slave3_tdata ;
      -- s_gtp_slave_tkeep_arr (3) <= S_gt0_slave3_tkeep ;
      -- s_gtp_slave_tlast_arr (3) <= S_gt0_slave3_tlast ;
      -- s_gtp_slave_tvalid_arr(3) <= S_gt0_slave3_tvalid;
      -- S_gt0_slave3_tready <= s_gtp_slave_tready_arr(3);
-----------------------------------------------------------------------
end Behavioral;